<?php
    /**
     * Memcache Helper
     *
     * @desc Module Parameters
     * server:      server name
     * port:        port
     * hostKey:		hostKey
     * @package Eaze
     * @subpackage Eaze.Modules
     * @static
     * @author Sergey Bykov
     */
    class MemcacheHelper implements IModule {

        /**
         * Total Get Requests
         *
         * @var int
         */
        public static $TotalGetRequests = 0;

        /**
         * Total Set Requests
         *
         * @var int
         */
        public static $TotalSetRequests = 0;

        /**
         * Initialized Flag
         *
         * @var boolean
         */
        public static $Initialized = false;

        /**
         * Is Active
         *
         * @var boolean
         */
        public static $Active = false;

        /**
         * Module Parameters
         *
         * @var array
         */
        private static $params = array();

        /**
         * Memcache Instance
         *
         * @var Memcache
         */
        private static $memcache ;

        /**
         * Current Host MD5 Key
         *
         * @var string
         */
        private static $keyPrefix;

        /**
         * Is Active
         *
         */
        public static function IsActive() {
            return self::$Active;
        }


        /**
         * Connect
         *
         */
        private static function connect() {
            if ( !self::$Active ) {
                return ;
            }

            // connect
            self::$memcache = memcache_connect( self::$params["server"], self::$params["port"] );

            if ( !self::$memcache ) {
                self::$Active = false;
            }
        }


        /**
         * Init Module
         *
         * @param DOMNodeList $params  the params node list
         * @static
         */
        public static function Init(DOMNodeList $params) {
            foreach ( $params as $param ) {
                self::$params[$param->getAttribute("name")] = $param->nodeValue;
            }

            if ( !isset( self::$params["server"] ) ) {
                self::$params["server"] = "localhost";
            }

            if ( !isset( self::$params["port"] ) ) {
                self::$params["port"] = "11211";
            }

            if ( empty( self::$params["active"] ) ||  "false" == self::$params["active"] ) {
                self::$params["active"] = false;
            } else {
                if ( class_exists( "Memcache") ) {
                    self::$params["active"] = true;
                } else {
                    self::$params["active"] = false;
                    Logger::Info( 'Memcache is not installed' );
                }
            }

            if ( !empty( self::$params["hostKey"] ) ) {
                self::$keyPrefix = self::$params["hostKey"];
            } else {
                self::$keyPrefix = Host::GetCurrentKey();
            }
            
            self::$Active      = self::$params["active"];
            self::$Initialized = true;

            self::connect();
        }


        /**
         * Get Value By Key From Cache 
         *
         * @param string|array $key
         * @return mixed
         */
        public static function Get( $key ) {
            if ( !self::$Active ) {
                return null;
            }

            /** if Key is Array add key prefix*/
            if ( is_array( $key ) ) {
            	foreach( $key as &$k ) {
            		$k = self::$keyPrefix . $k;
            	}
            } else {
            	$key = self::$keyPrefix . $key;
            }

            self::$TotalGetRequests ++;
            return self::$memcache->get( $key );
        }
        
        
        /**
         * Increment Key
         * 
         * @param string $key 
         */
        public static function Increment( $key ) {
			if ( !self::$Active ) {
                return null;
            }

            self::$TotalSetRequests ++;
                    	
        	return self::$memcache->increment( self::$keyPrefix . $key );
        }
        

        /**
         *  Add an item to the server
         * 
         * @param string 		$key 		key parameter
         * @param string 		$value		value
         * @param int			$flag 		Use MEMCACHE_COMPRESSED to store the item compressed (uses zlib). Default 0
         * @param int 			$expire		expiration date, default is 3600
         */
        public static function AddValue( $key, $value, $flag = 0, $expire = 3600 ) {
            if ( !self::$Active ) {
                return null;
            }

            self::$TotalSetRequests ++;
            return self::$memcache->add( self::$keyPrefix . $key, $value, $flag, $expire );
        }
        
        
        /**
         * Set Value
         *
         * @param string  		$namespace  namespace or tags
         * @param string 		$key 		key parameter
         * @param string 		$value		value
         * @param int			$flag 		Use MEMCACHE_COMPRESSED to store the item compressed (uses zlib). Default 0
         * @param int 			$expire		expiration date, default is 3600
         * @return mixed
         */
        public static function Set( $namespace, $key, $value, $flag = 0, $expire = 3600 ) {
            if ( !self::$Active ) {
                return null;
            }

            if ( !empty( $namespace ) ) {
            	self::AddKey( $key, $namespace, $flag, $expire );	
            }
            
            return self::$memcache->set( self::$keyPrefix . $key, $value, $flag, $expire );
        }


        /**
         * Replace Value
         *
         * @param array|string  $namespace  namespace or tags
         * @param string 		$key 		key parameter
         * @param string 		$value		value
         * @param int			$flag 		Use MEMCACHE_COMPRESSED to store the item compressed (uses zlib). Default 0
         * @param int 			$expire		expiration date, default is 3600
         * @return mixed
         */
        public static function Replace( $namespace, $key, $value, $flag = 0, $expire = 3600 ) {
            if ( !self::$Active ) {
                return null;
            }

            self::$TotalSetRequests ++;
            $result = self::$memcache->replace( self::$keyPrefix . $key, $value, $flag, $expire );

            if ( !$result ) {
                self::AddKey( $key, $namespace, $flag, $expire );

                self::$TotalSetRequests ++;
                $result = self::$memcache->add( self::$keyPrefix . $key, $value, $flag, $expire );
            }

            return $result;
        }


        /**
         * Delete Value By Key
         *
         * @param string $key
         * @param string $timeout  Execution time of the item. If it's equal to zero, the item will be deleted right away whereas if you set it to 30, the item will be deleted in 30 seconds.
         * @return mixed
         */
        public static function Delete( $key, $timeout = 0 ) {
            if ( !self::$Active ) {
                return null;
            }

            self::$TotalSetRequests ++;
            return self::$memcache->delete( self::$keyPrefix . $key, $timeout );
        }


        /**
         * Flush All Keys
         */
        public static function Flush() {
            if ( !self::$Active ) {
                return null;
            }

            return self::$memcache->flush();
        }


        
        /**
         * Get Keys By String Namespaces
         * 
         * @param string $namespace
         */
        public static function GetKeys( $namespace) {
            if ( !self::$Active ) {
                return null;
            }

            return self::Get( "keys_" . $namespace );
        }

        /**
         * Add Key to String Namespace
         *
         * @param string $key
         * @param string $namespace
         * @param string $flag			Use MEMCACHE_COMPRESSED to store the item compressed (uses zlib). Default 0
         * @param string $expire
         */
        public static function AddKey( $key, $namespace, $flag = 0, $expire = 3600 ) {
            if ( !self::$Active ) {
                return null;
            }

            $keys = self::Get( "keys_" . $namespace );
            $keys[$key] = $key;

            self::$TotalSetRequests ++;
            $result = self::$memcache->replace( self::$keyPrefix . "keys_" . $namespace, $keys, $flag, $expire );

            if ( !$result ) {
                self::$TotalSetRequests ++;
                $result = self::$memcache->add( self::$keyPrefix . "keys_" . $namespace, $keys, $flag, $expire );
            }
        }


        /**
         * Delete Data By String Namespace
         *
         * @param string $namespace
         * @return null
         * @deprecated
         */
        public static function DeleteByNamespace( $namespace ) {
            if ( !self::$Active ) {
                return null;
            }

            $keys = self::Get( "keys_" . $namespace );
            if ( !empty( $keys ) ) {
                self::$TotalSetRequests ++;
                self::$memcache->delete( self::$keyPrefix . "keys_" . $namespace );

                foreach ( $keys as $key ) {
                    self::$TotalSetRequests ++;
                    self::$memcache->delete( self::$keyPrefix . $key );
                }
            }
        }
        
        
        /**
         * Compare Tag Versions
         * 
         * @param array $tags1
         * @param array $tags2
         * @return bool true if equals
         */
        public static function CompareTags( $tags1, $tags2 ) {
            if ( count( $tags1) != count( $tags2 ) ) {
                return false;
            }

            if ( empty( $tags1 ) ) {
                $tags1 = array();
            }

            if ( empty( $tags2 ) ) {
                $tags2 = array();
            }
            
            $result  = true;
            foreach ( $tags1 as $tag1Key => $tag1Value ) {
                if ( empty( $tags2[$tag1Key] ) || $tags2[$tag1Key] != $tag1Value ) {
                    $result = false;
             		break;
                }
            }
            
            return $result;
        }


        /**
         * Get Memcached Version
         * @return string
         */
        public static function GetVersion() {
            if ( !self::$Active ) {
                return null;
            }

            return self::$memcache->getVersion();
        }

        /**
         * Get Memcached Server Stats
         * @return string
         */
        public static function GetStats() {
            if ( !self::$Active ) {
                return null;
            }

            return self::$memcache->getStats();
        }


        /**
         * Get Key Prefix
         * @return string
         */
        public static function GetKeyPrefix() {
            return self::$keyPrefix;
        }
    }
?>